@@ -1,16 +1,14 @@
 from os import stat
+import based58
+from hashlib import sha256
+from bech32ref import segwit_addr
 from .link_extractors import UUF
 from urllib.parse import urljoin
-
 from bs4 import BeautifulSoup
-
 import re
-
 import nltk
-
 from .textan import TextAnalysis
 
-
 class Item():
     def __init__(self, value=None):
         self.value = value
@@ -37,8 +35,16 @@ class Keyword(Item):
     pass
 
 class BTC_Wallet(Item):
-    pass
-
+    @staticmethod
+    def isValid(address: str) -> bool:
+        if address[0] in ['1', '3']:
+            decode_address = based58.b58decode(address.encode('utf-8'))
+            return decode_address[-4:] == sha256(sha256(decode_address[:-4]).digest()).digest()[:4]
+        elif address.startswith("bc"):
+            hrpgot, data, spec = segwit_addr.bech32_decode(address)
+            return (hrpgot is not None) and (data is not None) and (spec is not None) 
+        else:
+            return False
 
 class ETH_Wallet(Item):
     pass
@@ -99,6 +105,10 @@ class Whatsapp_URL(Item):
 class Skype_URL(Item):
     pass
 
+
+class Discord_URL(Item):
+    pass
+
 class Paste(Item):
     pass
 
@@ -130,6 +140,8 @@ email_regex = r"([a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]{2,6})"
 
 btc_wallet_regex = r"([13][a-km-zA-HJ-NP-Z1-9]{25,34})"
 
+btc_wallet_bech32_regex = r"(bc1[qp][qpzry9x8gf2tvdw0s3jn54khce6mua7l]{38,58})"
+
 eth_wallet_regex = r"0x([0-9a-f]{40})"
 
 bitname_domain_regex = r'(?:[a-zA-Z0-9]+\.)+bit'
@@ -140,6 +152,8 @@ telegram_url_regex = r'((?:https?\:\/\/)?(?:t\.me|telegram\.me)(?:\/[a-zA-Z0-9_-
 
 whatsapp_url_regex = r'((?:https?\:\/\/)?chat\.whatsapp\.com(?:\/[a-zA-Z0-9_-]+)+)'
 
+discord_url_regex = r'((?:https?\:\/\/)?discord(?:app)?\.(?:gg|com|net)(?:\/[a-zA-Z0-9_-]+)+)'
+
 skype_url_regex = r'((?:https?\:\/\/)?join\.skype\.com(?:\/[a-zA-Z0-9]+)+)'
 
 username_regex = r"([a-zA-Z0-9\$\.,;_-]{8,20})[^a-zA-Z0-9]"
@@ -264,7 +278,7 @@ class reStalker():
                  location=False, organization=False, keyphrase=False,
                  keywords=[],
                  base64=False, own_name=False,
-                 whatsapp=False, telegram=False, skype=False,
+                 whatsapp=False, discord=False, telegram=False, skype=False,
                  md5=False, sha1=False, sha256=False,
                  all=False):
 
@@ -298,6 +312,7 @@ class reStalker():
         self.password = password or all
         self.base64 = base64 or all
         self.whatsapp = whatsapp or all
+        self.discord = discord or all
         self.telegram = telegram or all
         self.skype = skype or all
 
@@ -323,7 +338,8 @@ class reStalker():
                 pass
 
         try:
-            soup = BeautifulSoup(body, "html.parser")
+            # soup = BeautifulSoup(body, "html.parser")
+            soup = BeautifulSoup(body, "lxml")
             if soup:
                 links = soup.findAll('a')
                 if links:
@@ -386,17 +402,18 @@ class reStalker():
                     for leave in subtree.leaves():
                         yield Location(value=leave[0])
 
-        ta = TextAnalysis(body)
 
-        for k in self.keywords:
-            # TODO Generate k variations 
-            k = k.lower()
-            if ta.is_keyword_present(k) > 0 or body.lower().find(k) >= 0:
-                yield Keyword(value=k)
+        if len(self.keywords) > 0 or self.keyphrase:
+            ta = TextAnalysis(body)
+            for k in self.keywords:
+                # TODO Generate k variations 
+                k = k.lower()
+                if ta.is_keyword_present(k) > 0 or body.lower().find(k) >= 0:
+                    yield Keyword(value=k)
 
-        if self.keyphrase:
-            for k in ta.extract_top_keyphrases():
-                yield Keyphrase(value=k)
+            if self.keyphrase:
+                for k in ta.extract_top_keyphrases():
+                    yield Keyphrase(value=k)
 
         # TODO Test si el valor es None
         # TODO Refactor para iterar
@@ -415,9 +432,13 @@ class reStalker():
                     yield Username(value=email.split('@')[0])
 
         if self.btc_wallet:
-            btc_wallets = re.findall(btc_wallet_regex, body)
-            for btc_wallet in btc_wallets:
-                yield BTC_Wallet(value=btc_wallet)
+            for btc_wallet in re.findall(btc_wallet_regex, body):
+                if BTC_Wallet.isValid(address=btc_wallet):
+                    yield BTC_Wallet(value=btc_wallet)
+                    
+            for btc_wallet in re.findall(btc_wallet_bech32_regex, body):
+                if BTC_Wallet.isValid(address=btc_wallet):
+                    yield BTC_Wallet(value=btc_wallet)        
 
         if self.eth_wallet:
             eth_wallets = re.findall(eth_wallet_regex, body)
@@ -494,6 +515,16 @@ class reStalker():
                     link_item = link
                 yield Whatsapp_URL(value=link_item)
 
+        if self.discord:
+            discord_links = re.findall(discord_url_regex, body)
+            discord_links = extract_elements(discord_links)
+            for link in discord_links:
+                try:
+                    link_item = UUF(link).full_url
+                except Exception as e:
+                    link_item = link
+                yield Discord_URL(value=link_item)
+
         if self.telegram:
             telegram_links = re.findall(telegram_url_regex, body)
             for link in telegram_links:
@@ -553,16 +584,19 @@ class reStalker():
         i = 0
 
         chunk_size = buff_size//2
+
+        # print("Chunks", len(body)//chunk_size)
         
-        while i*buff_size <= len(body):
+        while i*chunk_size <= len(body):
             
-            chunk = body[i*buff_size:(i+2)*buff_size]
-            chunk_analysis = self._analyze_chunk(body, origin=origin)
+            chunk = body[i*chunk_size:(i+2)*chunk_size]
+            chunk_analysis = self._analyze_chunk(chunk, origin=origin)
             
             for result in chunk_analysis:
                 yield result
 
             i += 1
+            # print("Chunk", i)
             
 
 # import stalker
